@@ -877,8 +877,8 @@ void AbstractTaskItem::setBackgroundFadeAlpha(qreal progress)
 
 bool AbstractTaskItem::shouldIgnoreDragEvent(QGraphicsSceneDragDropEvent *event)
 {
-   if (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
-       event->mimeData()->hasFormat(TaskManager::Task::groupMimetype())) {
+    if (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
+        event->mimeData()->hasFormat(TaskManager::Task::groupMimetype())) {
         return true;
     }
 
@@ -889,13 +889,15 @@ bool AbstractTaskItem::shouldIgnoreDragEvent(QGraphicsSceneDragDropEvent *event)
         if (!uris.isEmpty()) {
             foreach (const QUrl &uri, uris) {
                 KUrl url(uri);
-                if (url.isLocalFile()) {
-                    const QString path = url.toLocalFile();
-                    QFileInfo info(path);
-                    if (info.isDir() || !info.isExecutable()) {
-                        return false;
-                        break;
-                    }
+                if (!url.isLocalFile()) {
+                    return false;
+                }
+
+                const QString path = url.toLocalFile();
+                QFileInfo info(path);
+                if (info.isDir() || !info.isExecutable()) {
+                    return false;
+                    break;
                 }
             }
 
@@ -916,19 +918,19 @@ void AbstractTaskItem::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
     event->accept();
 
     if (!m_activateTimerId) {
-        m_activateTimerId = startTimer(500);
+        m_activateTimerId = startTimer(250);
+        m_oldDragPos = event->pos();
     }
 }
 
 void AbstractTaskItem::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
 {
-    Q_UNUSED(event);
-
     // restart the timer so that activate() is only called after the mouse
     // stops moving
-    if (m_activateTimerId) {
+    if (m_activateTimerId && event->pos() != m_oldDragPos) {
+        m_oldDragPos = event->pos();
         killTimer(m_activateTimerId);
-        m_activateTimerId = startTimer(500);
+        m_activateTimerId = startTimer(250);
     }
 }
 
