@@ -21,8 +21,6 @@
 
 #include "battery.h"
 
-#include "brightnessosdwidget.h"
-
 #include <QApplication>
 #include <QDBusConnection>
 #include <QDBusInterface>
@@ -94,8 +92,7 @@ Battery::Battery(QObject *parent, const QVariantList &args)
       m_labelAnimation(0),
       m_acAlpha(0),
       m_acAnimation(0),
-      m_ignoreBrightnessChange(false),
-      m_brightnessOSD(0)
+      m_ignoreBrightnessChange(false)
 {
     //kDebug() << "Loading applet battery";
     setAcceptsHoverEvents(true);
@@ -127,7 +124,6 @@ Battery::Battery(QObject *parent, const QVariantList &args)
 
 Battery::~Battery()
 {
-    delete m_brightnessOSD;
 }
 
 void Battery::init()
@@ -167,7 +163,7 @@ void Battery::init()
         initPopupWidget();
         // let's show a brightness OSD
         QDBusConnection::sessionBus().connect("org.kde.Solid.PowerManagement", "/org/kde/Solid/PowerManagement", "org.kde.Solid.PowerManagement",
-                                              "brightnessChanged", this, SLOT(showBrightnessOSD(int)));
+                                              "brightnessChanged", this, SLOT(updateSlider(float)));
     }
 
     if (m_acAdapterPlugged) {
@@ -1106,27 +1102,4 @@ qreal Battery::acAlpha() const
     return m_acAlpha;
 }
 
-void Battery::showBrightnessOSD(int brightness)
-{
-    // code adapted from KMix
-    if (!m_brightnessOSD) {
-        m_brightnessOSD = new BrightnessOSDWidget();
-    }
-
-    m_brightnessOSD->setCurrentBrightness(brightness);
-    m_brightnessOSD->show();
-    m_brightnessOSD->activateOSD(); //Enable the hide timer
-
-    //Center the OSD
-    QRect rect = KApplication::kApplication()->desktop()->screenGeometry(QCursor::pos());
-    QSize size = m_brightnessOSD->sizeHint();
-    int posX = rect.x() + (rect.width() - size.width()) / 2;
-    int posY = rect.y() + 4 * rect.height() / 5;
-    m_brightnessOSD->setGeometry(posX, posY, size.width(), size.height());
-
-    if (m_extenderVisible && m_brightnessSlider) {
-        updateSlider(brightness);
-    }
-}
-
 #include "battery.moc"
